# Lengths (equivalents in meters)
meter = 1
inch = 0.0254
foot = 12 * inch
yard = 3 * foot
mile = 5280 * foot
naut_mile = 1852

# Masses (equivalents in kilograms)
kg = 1
gram = 0.001
slug = 14.59390
lbm = 0.45359237
short_ton = 2000 * lbm
long_ton = 2240 * lbm
oz = lbm / 16

# Time (equivalents in seconds)
second = 1
minute = 60
hour = 60 * minute
day = 86400
week = 7 * day
year = 365.25 * day
month = year / 12

# Force (equivalents in Newtons)
newton = 1
lbf = slug * foot

# Speed (equivalents in m/s)
knot = 1.852 * 1000 / hour
mph = mile / hour
fps = foot / 1

# Volume (equivalents in m^3)
liter = 0.001

# SI prefixes
yocto = 1e-24
zepto = 1e-21
atto = 1e-18
femto = 1e-15
pico = 1e-12
nano = 1e-9
micro = 1e-6
milli = 1e-3
centi = 1e-2
deci = 1e-1
deka = 1e1
hecto = 1e2
kilo = 1e3
mega = 1e6
giga = 1e9
tera = 1e12
peta = 1e15
exa = 1e18
zetta = 1e21
yotta = 1e24
